module Fastlane
  module Actions
    class UploadSymbolsToTracerAction < Action
      def self.run(options)
        FastlaneCore::PrintTable.print_values(config: options,
                                              title: "Tracer symbols upload")

        appName = options[:app_name]
        version = options[:build_version]
        token = options[:api_token]
        dsym_path = options[:dsym]

        if dsym_path && File.exist?(dsym_path)
          if File.directory?(dsym_path)
            UI.message("dSYM path is folder, zipping...")
            dsym_path = Actions::ZipAction.run(path: dsym_path, output_path: dsym_path + ".zip")
            UI.message("dSYM files zipped")
          end

          UI.message("Starting dSYM upload...")

          url = options[:url] || "https://api-hprof.odkl.ru"
          status, message, response = upload_file(url: url, appName: appName, version: version, token: token, file: dsym_path)
          case status
          when 0, 429
            if status == 0
              UI.error("Faraday http exception while upload: #{message}")
            else
              UI.error("Retryable error creating release upload #{status}: #{message}")
            end
          when 200...300
            UI.message("dSYM uploaded")
          when 400
            errorType = response.body['errorCode'] || "Unknown"
            errorCode = response.body['code'] || status
            errorMessage = response.body['message'] || message
            UI.error("Error type: #{errorType} code: #{errorCode} message: #{errorMessage}")
          when 401
            UI.user_error!("Auth Error, provided invalid token")
          when 404
            UI.error("Not found, invalid application name")
          when 413
            UI.error("File too large")
          when 500...600
            UI.abort_with_message!("Internal Service Error, please try again later")
          else
            UI.error("Error #{response.status}: #{response.body}")
          end
        else
          UI.user_error!("dSYM not found")
        end
      end

      def self.upload_file(url:, appName:, version:, token:, file:)
        require "faraday"
        require "faraday_middleware"

        method = "api/symbol/upload?symbolToken=#{token}"
        body = {
          versionName: appName,
          versionCode: version,
          file: Faraday::UploadIO.new(file, "application/octet-stream"),
        }

        UI.verbose("📡 Request #{url}/#{method} started")

        status = 0
        begin
          connection = Faraday.new(url: url) do |builder|
            builder.request(:multipart)
            builder.request(:url_encoded)
            builder.request(:retry, max: 3, interval: 5)
            builder.response(:json, content_type: /\bjson$/)
            builder.adapter(:net_http)
            builder.use(FaradayMiddleware::FollowRedirects)
          end

          response = connection.post(method) do |request|
            request.body = body
          end

          status = response.status
          message = response.body
          UI.verbose("📡 Request complete status:#{status} #{JSON.pretty_generate(message)}\n")
        rescue Faraday::Error => e
          status = 0
          message = e.message
          UI.verbose("📡 Request failed\nerror: #{status} #{JSON.pretty_generate(message)}\n")
        end

        return status, message, response
      end

      #####################################################
      # @!group Documentation
      #####################################################
      def self.description
        "Tracer symbols upload action"
      end

      def self.available_options
        [
          FastlaneCore::ConfigItem.new(key: :app_name,
                                       description: "App name registered in Tracer",
                                       optional: false,
                                       type: String,
                                       verify_block: proc do |value|
                                         UI.user_error!("No App name given, pass using `app_name: 'name'`") unless value && !value.empty?
                                       end),

          FastlaneCore::ConfigItem.new(key: :build_version,
                                       description: "Build version for app in Tracer",
                                       optional: false,
                                       type: String,
                                       verify_block: proc do |value|
                                         UI.user_error!("No build version given, pass using `build_version: 'version'`") unless value && !value.empty?
                                       end),

          FastlaneCore::ConfigItem.new(key: :api_token,
                                       description: "API Token for Tracer",
                                       optional: false,
                                       type: String,
                                       verify_block: proc do |value|
                                         UI.user_error!("No API token for Tracer given, pass using `api_token: 'token'`") unless value && !value.empty?
                                       end),

          FastlaneCore::ConfigItem.new(key: :dsym,
                                       description: "Path to your symbols file, folder or zip archive of dSyms",
                                       default_value: Actions.lane_context[SharedValues::DSYM_OUTPUT_PATH],
                                       optional: false,
                                       type: String,
                                       verify_block: proc do |value|
                                         UI.user_error!("Couldn't find dSYM file at path '#{value}'") unless File.exist?(value)
                                       end),

          FastlaneCore::ConfigItem.new(key: :url,
                                       description: "Trace API url",
                                       optional: true,
                                       type: String),
        ]
      end

      def self.authors
        ["OK.Tech"]
      end

      def self.is_supported?(platform)
        [:ios].include?(platform)
      end

      def self.example_code
        [
          'upload_symbols_to_tracer(
            app_name: "OK",
            build_version: "10.9",
            api_token: "356ad5ca9259f723904988c59f2ae1b0",
            dsym: "/Users/tayphoon/Desktop/dSYMs.zip"
          )',
          'upload_symbols_to_tracer(
            app_name: "OK",
            build_version: "10.9",
            api_token: "356ad5ca9259f723904988c59f2ae1b0",
            dsym: "/Users/tayphoon/Desktop/OK.dSYM"
          )',
          'upload_symbols_to_tracer(
            app_name: "OK",
            build_version: "10.9",
            api_token: "356ad5ca9259f723904988c59f2ae1b0",
          )',
        ]
      end
    end
  end
end
